function BackgroundStat_GaborS1

import feature.*

Upperbound = feature.getGaborResponseUpperbound; % saturation value
binsize = .2;  % binsize for computing F
scale = feature.getGaborScale;
norient = 16;
featureVersion = 'GaborS1';

imageSet = 'Train/neg';
allfilter = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
files = dir( sprintf('Feature/%s/%s/*.mat',featureVersion,imageSet) );
n = length(files);
n = min(n,100); % !! to fit into memory, we truncate negative examples
imsize = zeros(n,2);
for i = 1:n
    featurePath = sprintf('Feature/%s/%s/%05d.mat',featureVersion,imageSet,i);
    load(featurePath,'S1LocalNorm', 'M1LocalNorm','RANGE');
    S1LocalNorm = convert2float(S1LocalNorm,RANGE);
    feature.mexc_Sigmoid(Upperbound, S1LocalNorm);
    for o = 1:norient
        allfiltered{i,o} = single( S1LocalNorm{o} );
    end
    imsize(i,:) = size(allfiltered{i,1});
end


binnum = floor(Upperbound/binsize)+1;  % binnumbers
histog = zeros(binnum, 1);  % store F  

% MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
disp('start histogramming');
mexc_Histogram(n, norient, allfiltered, h, imsize(:,1), imsize(:,2), binsize, binnum, histog, Upperbound);
disp(['histogramming time: ' num2str(toc) ' seconds']);

% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
r = (0:(binnum-1))*binsize;

M = 50; 
e = zeros(M, 1);
Z = zeros(M, 1); 
for k=1:M
    lambda = (k-1.)/10.;
    p = exp(lambda*r).*(histog'); 
    Z(k) = sum(p*binsize); p = p/Z(k);
    %plot(r, p); 
    e(k) = sum(r.*p*binsize);
end
lam = (0:(M-1))/10.; 
lz = log(Z);

save(sprintf('Feature/naturalStat_GaborS1_LocalNorm.mat'), 'M',...
    'lam', 'e', 'lz', 'Upperbound', 'scale', 'norient', 'histog');

clear S1LocalNorm;


