function count = dumpFeature_GaborS1(posOrNeg, featureVersion,windowSize,imagePath,imageSet,allfilter,localHalfx,...
    Lrange, Orange,count)

disp(['extracting Gabor feature for : ' imagePath]);

import feature.*

% compute feature maps for a single image
norient = length(allfilter);
filterSize = size(allfilter{1},1);
halfFilterSize = floor(filterSize/2);
GaborSize = single(size(allfilter{1},1));
subsampleM1 = 1;
windowStep = feature.getWindowStep;
thresholdFactor = .01; % for local normalization

% parameters used for UINT8 storage
RANGE = [0 10];

im = imread(imagePath); % load image file
if size(im,3)==3 % color image
    im = rgb2gray(im); % convert to gray image
end
if size(im,1) * size(im,2) > 500 * 500
	im = imresize(im,500/sqrt(size(im,1)*size(im,2)));
end

srcImg = single(im); % convert to floating point (single precision) numbers

mask = ones(windowSize);
mask([1:halfFilterSize end-halfFilterSize+1:end],:) = 0;
mask(:,[1:halfFilterSize end-halfFilterSize+1:end]) = 0;


if posOrNeg > 0
	imsize = size(im);
	margin = floor( (imsize(1)-windowSize(1))/2 );
	top = margin + 1;
	margin = floor( (imsize(2)-windowSize(2))/2 );
	left = margin + 1;
	windows = {[top,top+windowSize(1)-1,left,left+windowSize(2)-1]};
else
	% window sampling
	windows = {};
	c = 0;
	for row = 0:windowStep:size(im,1)-windowSize(1)
		for col = 0:windowStep:size(im,2)-windowSize(2)
			c = 1 + c;
			windows{c} = [row+1,row+windowSize(1),col+1,col+windowSize(2)];
		end
	end
	disp(sprintf('%d windows',length(windows)));
end

for i = 1:length(windows)
	top = windows{i}(1);
	down = windows{i}(2);
	left = windows{i}(3);
	right = windows{i}(4);
	im = srcImg(top:down,left:right);
	
	S1 = applyfilterfftsame({im},allfilter); % raw S1 Map (no sigmoid transformation)
	
	localHalfy = localHalfx;

	% local normalization for sketch feature
	S1LocalNorm = cell(1,norient);
	for o = 1:norient
		S1LocalNorm{o} = single( S1{o} + 0 );
	end
	mexc_LocalNormalize(windowSize(1), windowSize(2), norient, halfFilterSize, localHalfx, localHalfy, S1LocalNorm, thresholdFactor);
	% set the boundary pixels to zero
	for o = 1:norient
		S1LocalNorm{o} = single( S1LocalNorm{o} .* mask );
	end

	% sigmoid transformation on S1LocalNorm (save it to the learning stage)
	% mexc_Sigmoid(Upperbound, S1LocalNorm);

	[M1LocalNorm ARGMAX1LocalNorm M1RowShift M1ColShift M1OriShifted] = ...
		mexc_ComputeMAX1( norient, S1LocalNorm, Lrange,...
			Orange, subsampleM1 );

	% store all the maps in uint8 with a linear scaling
	S1LocalNorm = feature.convert2uint8( S1LocalNorm, RANGE );
	M1LocalNorm = feature.convert2uint8( M1LocalNorm, RANGE );

	
	count = count + 1;
	filename = sprintf('%s/%s/%s/%05d.mat','Feature/',featureVersion,imageSet,count);
	thisWindow = windows{i};
	save(filename,...
		'windowSize','thisWindow','imagePath',...
		'GaborSize','S1LocalNorm',...
		'localHalfx','RANGE',... % for storing in uint8
		'M1LocalNorm','ARGMAX1LocalNorm',... % for MAX1 maps
		'M1RowShift', 'M1ColShift', 'M1OriShifted'); % feature map
end


