function count = dumpFeature_HoG(posOrNeg,featureVersion,...
	windowSize,imagePath,imageSet,count)
	
disp(['extracting HoG for : ' imagePath]);

sbin = feature.HoG_binsize;
windowStep = feature.getWindowStep;

im = imread(imagePath); % load image file

if size(im,3) < 3
	im = repmat(im,[1 1 3]);
end
im = double(im);
% resize if the image is too large
if size(im,1) * size(im,2) > 500 * 500
	im = imresize(im,500/sqrt(size(im,1)*size(im,2)));
end

if posOrNeg > 0
	imsize = size(im);
	margin = floor( (imsize(1)-windowSize(1))/2 );
	top = margin + 1;
	margin = floor( (imsize(2)-windowSize(2))/2 );
	left = margin + 1;
	windows = {[top,top+windowSize(1)-1,left,left+windowSize(2)-1]};
else
	% window sampling
	windows = {};
	c = 0;
	for row = 0:windowStep:size(im,1)-windowSize(1)
		for col = 0:windowStep:size(im,2)-windowSize(2)
			c = 1 + c;
			windows{c} = [row+1,row+windowSize(1),col+1,col+windowSize(2)];
		end
	end
	disp(sprintf('%d windows',length(windows)));
end

for i = 1:length(windows)
	top = windows{i}(1);
	down = windows{i}(2);
	left = windows{i}(3);
	right = windows{i}(4);
	patch = im(top:down,left:right,:);
	HoGmap = feature.mexc_ComputeHoG( patch, sbin );
	count = count + 1;
	filename = sprintf('%s/%s/%s/%05d.mat','Feature/',featureVersion,imageSet,count);
	thisWindow = windows{i};
	save(filename,...
		'thisWindow','imagePath',...
		'HoGmap', 'patch'); % feature map
end



