function dumpFeature_MSGabor(featureVersion,overwrite,rescale,windowSize,imagePath,allfilter,maxImArea,...
    Lrange_ratio, Orange, Upperbound)

disp(['extracting multiscale Gabor feature for : ' imagePath]);

import feature.*

% compute feature maps for a single image
norient = 16;
subsampleM1 = 1;
thresholdFactor = .01;

% parameters used for UINT8 storage
RANGE = [0 Upperbound];

% check if the file already exists
if exist( ...
        sprintf('%s/%s.mat','Feature/GaborS1',...
        imagePath((1+length('Image/'):end))),'file')
    if ~(overwrite > 0)
        return;
    end
end

im = imread(imagePath); % load image file
if size(im,3)==3 % color image
    im = rgb2gray(im); % convert to gray image
end


% resize image
if rescale > 0
    sx = size(im,1); sy = size(im,2);
    if sx >= sy
        newSx = windowSize;
        newSy = floor(.5+windowSize*sy/sx);
    else
        newSy = windowSize;
        newSx = floor(.5+windowSize*sx/sy);
    end
    im = imresize(im,[newSx,newSy],'nearest');
else
    if size(im,1) * size(im,2) > maxImArea
        im = imresize(im, sqrt(maxImArea/size(im,1)/size(im,2)),'nearest');
    end
    [newSx newSy] = size(im);
end




% compute S1 maps
im = single(im); % convert to floating point (single precision) numbers
im = im - mean(im(:)); % global image normalization
im = im / std(im(:));

nScale = numel(allfilter)/norient;
S1 = cell(1,numel(allfilter));
for s = 1:nScale
	S1(1,(s-1)*norient+(1:norient)) = applyfilterfftsame({im},allfilter((s-1)*norient+(1:norient))); % raw S1 Map (no sigmoid transformation)
end

S1LocalNorm = S1;
for j = 1:numel(S1)
    S1LocalNorm{j} = S1{j} + 0.;
end

% local normalization for sketch feature
for iScale = 1:nScale
    tmpfilter = allfilter{ (iScale-1)*norient+1 };
    halfFilterSize = floor(size(tmpfilter,1)/2);
    localHalfx = floor(size(tmpfilter,1)*1.5);
    mask = ones(size(S1LocalNorm{(iScale-1)*norient+1}));
    mask([1:halfFilterSize end-halfFilterSize+1:end],:) = 0;
    mask(:,[1:halfFilterSize end-halfFilterSize+1:end]) = 0;
    mexc_LocalNormalize(newSx, newSy, norient, halfFilterSize,...
        localHalfx, localHalfx, S1LocalNorm((iScale-1)*norient+(1:norient)), thresholdFactor);
    for o = 1:norient
        S1LocalNorm{(iScale-1)*norient+o} = S1LocalNorm{(iScale-1)*norient+o} .* mask;
    end
end

% sigmoid transformation on S1LocalNorm 
mexc_Sigmoid(Upperbound, S1LocalNorm);

M1LocalNorm = cell(size(S1LocalNorm));
ARGMAX1LocalNorm = cell(size(S1LocalNorm));
for iScale = 1:nScale
	Lrange = floor( Lrange_ratio * size(allfilter{(iScale-1)*norient+1},1) );
	[M1LocalNorm((iScale-1)*norient+(1:norient)) ...
        ARGMAX1LocalNorm((iScale-1)*norient+(1:norient))...
        M1RowShift{iScale} M1ColShift{iScale} M1OriShifted{iScale}] = ...
		mexc_ComputeMAX1( norient, S1LocalNorm((iScale-1)*norient+(1:norient)), Lrange,...
			Orange, subsampleM1 );
end


% store all the maps in uint8 with a linear scaling
S1LocalNorm = feature.convert2uint8( S1LocalNorm, RANGE );
M1LocalNorm = feature.convert2uint8( M1LocalNorm, RANGE );

save(sprintf('%s/%s/%s.mat','Feature/',featureVersion,imagePath((1+length('Image/'):end))),... % image and filter parameters
	'allfilter','maxImArea','windowSize',... % image and filter parameters
    'S1LocalNorm',...
	'RANGE',... % for storing in uint8
	'M1LocalNorm','ARGMAX1LocalNorm',... % for MAX1 maps
	'M1RowShift', 'M1ColShift', 'M1OriShifted'); % for MAX1 maps


%{
S1LocalMean = cell(1,numel(allfilter));
for i = 1:numel(allfilter)
	radius = floor(size(allfilter{i},1)*1.2);
	localAveFilter = ones(radius*2+1,radius*2+1,'single')/single(radius*2+1)/single(radius*2+1);
	S1LocalMean{i} = filter2(localAveFilter,S1{i},'same');
end

nScale = numel(allfilter)/norient;
S1LocalMeanOverOrientation = cell(1,nScale);
for iScale = 1:nScale
	S1LocalMeanOverOrientation{iScale} = zeros(newSx,newSy);
	for o = 1:norient
		S1LocalMeanOverOrientation{iScale} = S1LocalMeanOverOrientation{iScale} + 1/norient * S1LocalMean{(iScale-1)*norient+o};
	end
end
% sigmoid transformation on S1
mexc_Sigmoid(Upperbound, S1);
S1 = feature.convert2uint8( S1, RANGE );

save(sprintf('%s/%s/%s.mat','Feature/',featureVersion,imagePath((1+length('Image/'):end))),... % image and filter parameters
	'allfilter','maxImArea','windowSize',... % image and filter parameters
    'S1','S1LocalNorm','S1LocalMean','S1LocalMeanOverOrientation',...
	'RANGE',... % for storing in uint8
	'M1LocalNorm',... % for MAX1 maps
	'M1RowShift', 'M1ColShift', 'M1OriShifted'); % for MAX1 maps

%}

% a more verbose version:
%{
save(sprintf('%s/%s/%s.mat','Feature/',featureVersion,imagePath((1+length('Image/'):end))),... % image and filter parameters
	'GaborSize','maxImArea','windowSize',... % image and filter parameters
    'S1','S1LocalNorm','S1LocalMean',...
    'localHalfx','S1LocalMeanRaw','S1LocalMeanOverOrientation',...
	'RANGE',... % for storing in uint8
	'M1', 'M1LocalNorm',... % for MAX1 maps
	'ARGMAX1', 'ARGMAX1LocalNorm', 'ARGMAX1LocalNormPerOri',... % for MAX1 maps
	'M1RowShift', 'M1ColShift', 'M1OriShifted'); % for MAX1 maps
%}
%%
