function featureConvolution(featureVersion)
% Pre-computation of S1 maps. Save them on the hard drive.

windowSize = feature.getWindowSize; % size of template window (for positive patches


if strcmp(featureVersion,'GaborS1')
	scale = feature.getGaborScale;
	featureConvolution_Gabor(featureVersion,scale,windowSize);
elseif strcmp(featureVersion,'MSGabor') % multiscale gabor
	feature.featureConvolution_MSGabor(featureVersion,overwrite,windowSize,maxImArea);
elseif strcmp(featureVersion,'HoG')
	featureConvolution_HoG(featureVersion,windowSize);
end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function featureConvolution_Gabor(featureVersion,scale,windowSize)

import feature.*
maxNeg = 15000;

norient = 16; % number of Gabor orientations
Lrange = getGaborLocationShiftLimit; % local deformation parameters (default: 4)
Orange = 1; 

% all positive images are inscribed in a square
allfilter = makefilter(scale,norient);

% size of local average operator:
localHalfx = floor(size(allfilter{1},1)*1.5); % 2.5 is the same as 2, 0.8 is not as good as 2

%% positive example images
% use the central window per example

count = 0;
imageSet = 'Train/pos'
if ~exist(sprintf('Feature/%s/%s',featureVersion,imageSet),'dir')
    mkdir(sprintf('Feature/%s/%s',featureVersion,imageSet));
end
imageNames = dir( sprintf('Image/%s/*.png',imageSet) );
imageNames = [imageNames;dir(sprintf('Image/%s/*.jpg',imageSet))];
n = length(imageNames);
for i = 1:n
    imagePath = sprintf('Image/%s/%s',imageSet,imageNames(i).name);
    posOrNeg = 1;
    count = dumpFeature_GaborS1(posOrNeg,featureVersion,windowSize,imagePath,imageSet,allfilter,localHalfx,...
		Lrange, Orange, count);
end

count = 0;
imageSet = 'Train/neg'
if ~exist(sprintf('Feature/%s/%s',featureVersion,imageSet),'dir')
    mkdir(sprintf('Feature/%s/%s',featureVersion,imageSet));
end
imageNames = dir( sprintf('Image/%s/*.png',imageSet) );
imageNames = [imageNames;dir(sprintf('Image/%s/*.jpg',imageSet))];
n = length(imageNames);
for i = 1:n
    imagePath = sprintf('Image/%s/%s',imageSet,imageNames(i).name);
    posOrNeg = -1;
    count = dumpFeature_GaborS1(posOrNeg,featureVersion,windowSize,imagePath,imageSet,allfilter,localHalfx,...
		Lrange, Orange, count);
	if count > maxNeg
		break;
	end
end

count = 0;
imageSet = 'Test/pos'
if ~exist(sprintf('Feature/%s/%s',featureVersion,imageSet),'dir')
    mkdir(sprintf('Feature/%s/%s',featureVersion,imageSet));
end
imageNames = dir( sprintf('Image/%s/*.png',imageSet) );
imageNames = [imageNames;dir(sprintf('Image/%s/*.jpg',imageSet))];
n = length(imageNames);
for i = 1:n
    imagePath = sprintf('Image/%s/%s',imageSet,imageNames(i).name);
    posOrNeg = 1;
    count = dumpFeature_GaborS1(posOrNeg,featureVersion,windowSize,imagePath,imageSet,allfilter,localHalfx,...
		Lrange, Orange, count);
end

count = 0;
imageSet = 'Test/neg'
if ~exist(sprintf('Feature/%s/%s',featureVersion,imageSet),'dir')
    mkdir(sprintf('Feature/%s/%s',featureVersion,imageSet));
end
imageNames = dir( sprintf('Image/%s/*.png',imageSet) );
imageNames = [imageNames;dir(sprintf('Image/%s/*.jpg',imageSet))];
n = length(imageNames);
for i = 1:n
    imagePath = sprintf('Image/%s/%s',imageSet,imageNames(i).name);
    posOrNeg = -1;
    count = dumpFeature_GaborS1(posOrNeg,featureVersion,windowSize,imagePath,imageSet,allfilter,localHalfx,...
		Lrange, Orange, count);
	if count > maxNeg
		break;
	end
end


%%

function featureConvolution_HoG(featureVersion,windowSize)

import feature.*
maxNeg = 15000;

count = 0;
imageSet = 'Train/pos'
if ~exist(sprintf('Feature/%s/%s',featureVersion,imageSet),'dir')
    mkdir(sprintf('Feature/%s/%s',featureVersion,imageSet));
end
imageNames = dir( sprintf('Image/%s/*.png',imageSet) );
imageNames = [imageNames;dir(sprintf('Image/%s/*.jpg',imageSet))];
n = length(imageNames);
for i = 1:n
    imagePath = sprintf('Image/%s/%s',imageSet,imageNames(i).name);
    posOrNeg = 1;
    count = dumpFeature_HoG(posOrNeg,featureVersion,windowSize,imagePath,imageSet,count);
end

count = 0;
imageSet = 'Train/neg'
if ~exist(sprintf('Feature/%s/%s',featureVersion,imageSet),'dir')
    mkdir(sprintf('Feature/%s/%s',featureVersion,imageSet));
end
imageNames = dir( sprintf('Image/%s/*.png',imageSet) );
imageNames = [imageNames;dir(sprintf('Image/%s/*.jpg',imageSet))];
n = length(imageNames);
for i = 1:n
    imagePath = sprintf('Image/%s/%s',imageSet,imageNames(i).name);
    posOrNeg = -1;
    count = dumpFeature_HoG(posOrNeg,featureVersion,windowSize,imagePath,imageSet,count);
    if count > maxNeg
		break;
	end
end

count = 0;
imageSet = 'Test/pos'
if ~exist(sprintf('Feature/%s/%s',featureVersion,imageSet),'dir')
    mkdir(sprintf('Feature/%s/%s',featureVersion,imageSet));
end
imageNames = dir( sprintf('Image/%s/*.png',imageSet) );
imageNames = [imageNames;dir(sprintf('Image/%s/*.jpg',imageSet))];
n = length(imageNames);
for i = 1:n
    imagePath = sprintf('Image/%s/%s',imageSet,imageNames(i).name);
    posOrNeg = 1;
    count = dumpFeature_HoG(posOrNeg,featureVersion,windowSize,imagePath,imageSet,count);
end

count = 0;
imageSet = 'Test/neg'
if ~exist(sprintf('Feature/%s/%s',featureVersion,imageSet),'dir')
    mkdir(sprintf('Feature/%s/%s',featureVersion,imageSet));
end
imageNames = dir( sprintf('Image/%s/*.png',imageSet) );
imageNames = [imageNames;dir(sprintf('Image/%s/*.jpg',imageSet))];
n = length(imageNames);
for i = 1:n
    imagePath = sprintf('Image/%s/%s',imageSet,imageNames(i).name);
    posOrNeg = -1;
    count = dumpFeature_HoG(posOrNeg,featureVersion,windowSize,imagePath,imageSet,count);
    if count > maxNeg
		break;
	end
end




