function learn_vs_all(LearningMethod,iRun,categories)
% model.ActiveBasis.learn_vs_all
% training a classifier against other object categories

toRelearn = false;

addpath Code/liblinear/matlab

load('Feature/naturalStat_GaborS1_LocalNorm.mat');
testOnlyOneCategory = false;
numStoredPoint = M;
scale = feature.getGaborScale;
nOri = 16;
norient = nOri;
Upperbound = feature.getGaborResponseUpperbound;
modelName = 'ActiveBasis';
[allFilter allSymbol] = feature.makefilter(scale,nOri);
halfFilterSize = floor( size(allFilter{1},1) / 2 );
epsilon = .1;
Correlation = feature.corr(allFilter, epsilon);

nCat = length(categories);
numElement = 300;
windowSize = feature.getWindowSize;
subsample = 1;
Lrange = feature.getGaborLocationShiftLimit;
Orange = 1;
if strcmp(LearningMethod,'lrl2')
	regularization_constant = .1; % for logistic regression
elseif strcmp(LearningMethod,'lrl1')
	regularization_constant = 10;
end
featureVersion = 'GaborS1';

% make sure the directory that contains the learned model is alrady created
for iCat = 1:nCat
	if ~exist(sprintf('Result/%s/%s/',categories{iCat},modelName),'dir')
		mkdir(sprintf('Result/%s/%s/',categories{iCat},modelName));
	end
end

%% learn active basis model
disp('learning active basis models ...');

for iCat = 1:nCat
	if testOnlyOneCategory
		if iCat > 1
			break;
		end
	end
	filename = sprintf( 'Result/%s/%s/abtemplate.mat', categories{iCat}, modelName );
	if toRelearn || ~exist(filename,'file')
		file0 = sprintf('Split/%s/split%d.mat',categories{iCat},iRun);
		load(file0,'posTrainImages');
		nimage = length(posTrainImages);
		S1Learn = cell(nimage,nOri);
		M1Learn = cell(nimage,nOri);
		ARGMAX1Learn = cell(nimage,nOri);
		for i = 1:nimage
			load(sprintf('Feature/%s/%s/%s.mat',...
		        featureVersion,categories{iCat},posTrainImages{i}),...
		        'S1LocalNorm','M1RowShift','M1ColShift','M1OriShifted','RANGE');
		    S1LocalNorm = feature.convert2float(S1LocalNorm,RANGE);
			[M1LocalNorm ARGMAX1LocalNorm M1RowShift M1ColShift M1OriShifted] = ...
				feature.mexc_ComputeMAX1( norient, S1LocalNorm, Lrange,...
					Orange, subsample );

			% inscribe if needed
			if size(M1LocalNorm{1},1) ~= windowSize || size(M1LocalNorm{1},2) ~= windowSize
				for o = 1:nOri
					M1LocalNorm{o} = single( util.inscribe( [windowSize, windowSize], M1LocalNorm{o}, 0 ) );
					S1LocalNorm{o} = single( util.inscribe( [windowSize, windowSize], S1LocalNorm{o}, 0 ) );
					ARGMAX1LocalNorm{o} = int32( util.inscribe( [windowSize, windowSize], ARGMAX1LocalNorm{o}, 0 ) );
				end
			end
		
			S1Learn(i,:) = S1LocalNorm(:);
			M1Learn(i,:) = M1LocalNorm(:);
			ARGMAX1Learn(i,:) = ARGMAX1LocalNorm(:);
		end
		
		
		selectedx = zeros(numElement,1);
		selectedy = zeros(numElement,1);
		selectedOrient = zeros(numElement,1);
		selectedLambda = zeros(numElement,1);
		selectedLogZ = zeros(numElement,1);
		commonTemplate = zeros(windowSize,windowSize,'single');
		deformedTemplate = cell(nimage,1);
		for i = 1:nimage
			deformedTemplate{i} = zeros(windowSize,windowSize,'single');
		end
		
		model.ActiveBasis.mexc_SharedSketch(nOri, Lrange, Orange, subsample, numElement ... % about active basis  
			, nimage, windowSize, windowSize, ...
			S1Learn, M1Learn, ARGMAX1Learn, ... % about training images 
			halfFilterSize, Correlation, allSymbol(1, :), ... % about filters
			numStoredPoint, lam, e, lz, ... % about exponential model 
			selectedOrient, selectedx, selectedy, selectedLambda, selectedLogZ, ... % learned parameters
			commonTemplate, deformedTemplate,... % learned templates 
			M1RowShift, M1ColShift, M1OriShifted); % stored local shifts
	
	
		save( filename, 'Lrange', 'Orange', 'numElement', 'windowSize', 'commonTemplate', ...
			'selectedx', 'selectedy', 'selectedOrient', 'selectedLambda', 'selectedLogZ' );
	end
end

%% distriminative Learning

% load all active basis templates
for iT = 1:nCat % each template
	if testOnlyOneCategory
		if iT > 1
			break;
		end
	end
	% load this template
	filename = sprintf( 'Result/%s/%s/abtemplate.mat', categories{iT}, modelName );
	load( filename, 'numElement', 'windowSize', 'commonTemplate', ...
		'selectedx', 'selectedy', 'selectedOrient' );
	allSelectedx{iT} = selectedx;
	allSelectedy{iT} = selectedy;
	allSelectedOrient{iT} = selectedOrient;
end

% collect feature responses from training positives and negatives; this is done for allh learned templates
nT = nCat; % number of learned templates == number of categories
X = cell(nT,1);
if testOnlyOneCategory
	X = cell(1,1);
end
catNo = [];
disp('collecting training data...');

count = 0;
for iCat = 1:nCat
	disp(['processing ' categories{iCat}]);
	file0 = sprintf('Split/%s/split%d.mat',categories{iCat},iRun);
	load(file0,'posTrainImages');
	nimage = length(posTrainImages);
	for i = 1:nimage
		load(sprintf('Feature/%s/%s/%s.mat',...
	        featureVersion,categories{iCat},posTrainImages{i}),...
	        'M1LocalNorm','ARGMAX1LocalNorm','M1RowShift','M1ColShift','M1OriShifted','RANGE');
	    
		count = count + 1;
		catNo(count,1) = iCat;
		model.ActiveBasis.load_feature;
		
		% fetch the corresponding feature responses
		for iT = 1:nT
			if testOnlyOneCategory
				if iT > 1
					break;
				end
			end
			
			if size(X{iT},1) == 0
				D = length(feavec{iT});
				X{iT} = zeros( nCat * nimage, D );
			end
			X{iT}(count,:) = feavec{iT}; 
		end
	end
	
end

disp('discriminative adjustment');

model.ActiveBasis.discriminativeAdjustment;





