function test_vs_all(LearningMethod,iRun,categories)
% model.ActiveBasis.test_vs_all - testing on positive images and negative examples of other categories.
%
% LearningMethod: e.g. 'lrl2','svml2','svmlight'
% iRun: the number of cross validation run.
% categories: the models (organized by category) to test.

modelName = 'ActiveBasis';
windowSize = feature.getWindowSize;
featureVersion = 'GaborS1';
testOnlyOneCategory = false;
Upperbound = feature.getGaborResponseUpperbound;

nCat = length(categories);

% make sure the result folder is created
for iCat = 1:nCat
	if ~exist(sprintf('Result/%s/%s',categories{iCat},modelName),'dir')
		mkdir(sprintf('Result/%s/%s',categories{iCat},modelName));
	end
end

% load all the trained models
weights = [];
for iCat = 1:nCat
	if testOnlyOneCategory
		if iCat > 1
			break;
		end
	end
	if strcmp(LearningMethod(1:2),'lr')
		load(sprintf('Result/%s/%s/model_vs_all_%s_run%d.mat',categories{iCat},modelName,LearningMethod,iRun),'lr_model');
		weights = [weights,lr_model.w(:)];
		clear lr_model;
	elseif strcmp(LearningMethod(1:3),'svm')
		load(sprintf('Result/%s/%s/model_vs_all_%s_run%d.mat',categories{iCat},modelName,LearningMethod,iRun),'svm_model');
		weights = [weights,svm_model.w(:)];
		clear svm_model;
	end
end

% test on multi-categories
fprintf(1,'testing ---- \n');
tic

nT = nCat;
% load all active basis templates
for iT = 1:nCat % each template
	if testOnlyOneCategory
		if iT > 1
			break;
		end
	end
	% load this template
	filename = sprintf( 'Result/%s/%s/abtemplate.mat', categories{iT}, modelName );
	load( filename, 'numElement', 'windowSize', 'commonTemplate', ...
		'selectedx', 'selectedy', 'selectedOrient' );
	allSelectedx{iT} = selectedx;
	allSelectedy{iT} = selectedy;
	allSelectedOrient{iT} = selectedOrient;
end
		
for iCat = 1:nCat
	% load images from this category, and test all templates on them
	disp(['processing ' categories{iCat}]);
	% collect active basis feature responses
	file0 = sprintf('Split/%s/split%d.mat',categories{iCat},iRun);
    load(file0,'posTestImages');
    nimage = length(posTestImages);
	
	scores = zeros(nimage,nCat);
	if testOnlyOneCategory
		scores = zeros(nimage,1);
	end
	
    for i = 1:nimage
    	load(sprintf('Feature/%s/%s/%s.mat',...
		        featureVersion,categories{iCat},posTestImages{i}),...
		        'M1LocalNorm','ARGMAX1LocalNorm','M1RowShift','M1ColShift','M1OriShifted','RANGE'); % there is no difference using S1LocalNorm and filter another time
		nOri = length(M1LocalNorm); %
		
		model.ActiveBasis.load_feature;
		
		for iT = 1:nCat % each template
			if testOnlyOneCategory
				if iT > 1
					break;
				end
			end
		    scores(i,iT) = [feavec{iT}(:)',1] * weights(:,iT);
		end
    end
    
    % save the scores of images for this category
    out_file = sprintf('Result/%s/%s/scores_vs_all_%s_run%d.mat',categories{iCat},modelName,LearningMethod,iRun);
	save( out_file, 'scores', 'weights' );
end


fprintf(1,'takes %.1f sec\n',toc);
