Y = [ones(size(Xpos,1),1);-1*ones(size(Xneg,1),1)];
X = [Xpos;Xneg];
disp(sprintf('data matrix: %d by %d',size(X,1),size(X,2)));

%% training
disp(sprintf('learning %s with %s:',modelName,LearningMethod));
if strcmp(LearningMethod(1:2),'lr') % logistic regression
	if strcmp(LearningMethod(end),'2') % L2 regularization
		param_s = 0;
	else % L1 regularization
		param_s = 6;
	end
	param_B = 1;
	param_c = regularization_constant;
    weight_of_pos = 1; %double(sum(y<0))/double(sum(y>0));
    param_e = 1e-2; % epsilon
    % -s 0 is for L2-logistic regression, -B 1 is for bias term, -c is for tuning parameter
	para=sprintf('-s %d -B %d -c %f -w1 %f -q -e %f',param_s,param_B,param_c,weight_of_pos,param_e); 
	
	%{
	% re-organize training data
	%	lrtrain uses the label of the first training example as the positive label (no matter its value is +1 or -1)
	%
	posind = find(Y>0); negind = find(Y<0);
	X = [X(posind,:);X(negind,:)]; Y = [Y(posind);Y(negind)];
	%}
	lr_model = lrtrain(Y,sparse(X),para);
	train_scores = [X ones(size(X,1),1)] * lr_model.w(:);
	model_file = sprintf( 'Result/%s/model_%s.mat', modelName, LearningMethod );
	save( model_file, 'lr_model', 'param_s', 'param_B', 'param_c', 'weight_of_pos', 'para', 'train_scores', 'Y' );

elseif strcmp(LearningMethod(1:6),'libsvm')
	
	param_s = 2;
	param_B = 1;
	param_c = .01;
    weight_of_pos = 1; %double(sum(y<0))/double(sum(y>0));
    param_e = 1e-2; % epsilon
    % -s 0 is for L2-logistic regression, -B 1 is for bias term, -c is for tuning parameter
	para=sprintf('-s %d -B %d -c %f -w1 %f -q -e %f',param_s,param_B,param_c,weight_of_pos,param_e); 
	
	model = lrtrain(Y,sparse(X),para);
	train_scores = [X ones(size(X,1),1)] * model.w(:);
	model_file = sprintf( 'Result/%s/model_%s.mat', modelName, LearningMethod );
	save( model_file, 'model', 'param_s', 'param_B', 'param_c', 'weight_of_pos', 'para', 'train_scores', 'Y' );
	
elseif strcmp(LearningMethod,'svmlight-sc') % svm-light, single class

	modelfile = sprintf('Result/model.txt');
    examplefile = sprintf('Result/exampletrain.txt');
    % output the training file
    fid = fopen(examplefile, 'w');
    for j = 1:length(Y)
        fprintf(fid, '%d ', Y(j));
        for k = 1:size(X,2)
            fprintf(fid, '%d:%.5f ', k, X(j,k));
        end
        fprintf(fid,'\r\n');
    end
    fclose(fid);
    
    para = '-v 0';
    system( sprintf('./Code/svmlight/svm_learn %s %s %s',para,examplefile,modelfile) );
    
    % load svm model
	fid = fopen(modelfile, 'r');
	% read line 10: number of SV
	for line = 1:9
		tline = fgetl(fid);
	end
	nSV = fscanf(fid, '%d', 1) - 1; % line 10 
	tline = fgetl(fid); 
	b = fscanf(fid, '%f', 1); % line 11
	tline = fgetl(fid);  
	D = size(X,2);
	svm_model.w = zeros(1,D+1);
	svm_model.w(D+1) = b;

	% read line 12~end: SV's
	for jSV = 1:nSV
		% read alpha * y
		alpha = fscanf( fid, '%f', 1 );
		vec = zeros(1,D);
		for d = 1:D
			feaID = fscanf( fid, '%d', 1 );
			colon = fscanf( fid, '%c', 1 );
			feaVal = fscanf( fid, '%f', 1 );
			vec(d) = feaVal;
		end
		svm_model.w(1:D) = svm_model.w(1:D) + vec * alpha;
		tline = fgetl(fid);
	end

	train_scores = [X ones(size(X,1),1)] * svm_model.w(:);

	model_file = sprintf( 'Result/%s/model_%s.mat', modelName, LearningMethod );
	save( model_file, 'svm_model', 'para', 'train_scores', 'Y' );

end
