function learn(LearningMethod)
% model.HIT.learn
% training a classifier against other object categories

smallTraingSize = true;
toRelearn = 1;
toReloadFeature = 1;

addpath Code/liblinear/matlab

load('Feature/naturalStat_GaborS1_LocalNorm.mat');

numStoredPoint = M;
scale = feature.getGaborScale;
nOri = 16;
norient = nOri;
Upperbound = feature.getGaborResponseUpperbound;
modelName = 'HIT';
[allFilter allSymbol] = feature.makefilter(scale,nOri);
halfFilterSize = floor( size(allFilter{1},1) / 2 );
epsilon = .1;
Correlation = feature.corr(allFilter, epsilon);

numImageForFeaturePursuit = 100;
numElement = 150; % default 100
windowSize = feature.getWindowSize;
subsample = .5;
Lrange = feature.getGaborLocationShiftLimit;
Orange = 1;
if strcmp(LearningMethod,'lrl2')
	regularization_constant = 1; % for logistic regression, default 0.1
elseif strcmp(LearningMethod,'lrl1')
	regularization_constant = 10;
end
featureVersion = 'GaborS1';

% make sure the directory that contains the learned model is alrady created
if ~exist(sprintf('Result/%s/',modelName),'dir')
	mkdir(sprintf('Result/%s/',modelName));
end

%% learn Hybrid Image Templates
disp('learning hybrid image templates ...');

	
filename = sprintf( 'Result/%s/hit.mat', modelName );
if toRelearn || ~exist(filename,'file')
	imageSet = 'Train/pos';
	files = dir( sprintf('Feature/%s/%s/*.mat',featureVersion,imageSet) );
	nimage = length(files);
	nimage = min(nimage,numImageForFeaturePursuit);
	
	model.HIT.learn_sketch;
	model.HIT.learn_texture;
    model.HIT.learn_flatness;
    model.HIT.learn_color;
    
	save( filename, 'Lrange', 'Orange', 'windowSize',...
        'HIT_sketch', 'HIT_texture', 'HIT_flatness', 'HIT_color' );
           
    load( filename, 'Lrange', 'Orange', 'windowSize',...
        'HIT_sketch', 'HIT_texture', 'HIT_flatness', 'HIT_color' );
else
	load( filename, 'Lrange', 'Orange', 'windowSize',...
        'HIT_sketch', 'HIT_texture', 'HIT_flatness', 'HIT_color' );
end


%% distriminative Learning

%% collect feature responses from training positives
disp('collecting training data...');

if toReloadFeature

	Xpos = [];
	imageSet = 'Train/pos';
	files = dir( sprintf('Feature/%s/%s/*.mat',featureVersion,imageSet) );
	n = length(files);
	if smallTraingSize
		n = min(n,100);
	end
	for i = 1:n
		featurePath = sprintf('Feature/%s/%s/%05d.mat',featureVersion,imageSet,i);
		load(featurePath,'S1LocalNorm', 'M1LocalNorm','RANGE');
		featurePath2 = sprintf('Feature/%s/%s/%05d.mat','HoG',imageSet,i); % to get the image patch
		load(featurePath2,'patch');
		
		model.HIT.load_feature;
		if size(Xpos,1) == 0
			D = length(feavec{1});
			disp(sprintf('feature dimemsion: %d',D));
			Xpos = zeros(n,D);
			Xpos(1,:) = feavec{1}(:);
		else
			Xpos(i,:) = feavec{1}(:);
		end
	end

	disp('neg');

	Xneg = [];
	imageSet = 'Train/neg';
	files = dir( sprintf('Feature/%s/%s/*.mat',featureVersion,imageSet) );
	n = length(files);
	if smallTraingSize
		n = min(n,1000);
	else
		n = min(n,10000);
	end

	for i = 1:n
		featurePath = sprintf('Feature/%s/%s/%05d.mat',featureVersion,imageSet,i);
		load(featurePath,'S1LocalNorm', 'M1LocalNorm','RANGE');
		model.HIT.load_feature;
		if size(Xneg,1) == 0
			D = length(feavec{1});
			Xneg = zeros(n,D);
			Xneg(1,:) = feavec{1}(:);
		else
			Xneg(i,:) = feavec{1}(:);
		end
	end

	feature_cache = sprintf('Result/%s/train_fea.mat',modelName);
	save(feature_cache,'Xpos','Xneg','D');
	
else
	
	feature_cache = sprintf('Result/%s/train_fea.mat',modelName);
	load(feature_cache,'Xpos','Xneg','D');

end

disp('discriminative adjustment');

model.HIT.discriminativeAdjustment;





