/* extract feature vector */

#include <math.h>
#include "mex.h"
#include <string.h>


/* global variables */
const float** S1map, **M1map, **colorIm;
const float* meanS1map;
int numOrient, imHeight, imWidth;
const mxArray* all_HIT_sketch, *all_HIT_texture, *all_HIT_flatness, *all_HIT_color;
int nT; /* number of templates */
float** vec; // it is actually a list of feature vectors for multiple templates
int* nF_HIT;
int* nF_sketch;
int* nF_texture;
int* nF_flatness;
int* nF_color;
int nBinColor;
int colorHistVersion;
int display; /* for debug */

bool use_L2_as_texure_feature = false;

/* getting the index of matlab image, (x,y) location, (sx, sy) sizes */
inline int px(int x, int y, int bx)    
{
   return (x + (y-1)*bx - 1);
}

void poolOriHist(int top, int bottom, int left, int right, float* oH, const float** fI)
{
    int ind,x,y;
    float sum;
    for(ind = 0; ind < numOrient; ind++)
    {
        for(x=top;x<=bottom;x++)
            for(y=left;y<=right;y++)
            {
            	int here = px(x,y,imHeight);
                oH[ind] += fI[ind][here];
            }
    }
    /* normalize the gradient histogram */
    sum = 0;
    for(ind = 0; ind < numOrient; ind++)
        sum += oH[ind];
    sum += 1e-10;
    if(sum>0)
        for(ind = 0; ind < numOrient; ind++)
            oH[ind] /= sum;
}

void poolColorHist(int top, int bottom, int left, int right, float* cH, const float** fI, int version)
{
    int ind,x,y;
    float sum;
		
	if( version == 0 )
	{
		for(ind = 0; ind < nBinColor; ind++)
		{
			int count = 0;
			for(x=top;x<=bottom;x++)
				for(y=left;y<=right;y++)
				{
					// mexPrintf("x=%d,y=%d,ind=%d,nBinColor=%d,version=%d\n",x,y,ind,nBinColor,colorHistVersion);
					cH[ind] += fI[ind][x+y*imHeight];
					++count;
				}
			cH[ind] /= (float)count * 10;
		}
	}
	else if( version == 1 )
	{
		// compute the 24-dimensional histogram
		
		/* normalize the histograms */
		sum = 0;
		for(ind = 0; ind < nBinColor; ind++)
			sum += cH[ind];
		if(sum>0)
			for(ind = 0; ind < nBinColor; ind++)
				cH[ind] /= (sum + 1e-10);
	}

}

void Compute()
{
    // count the total number of features per template
    nF_HIT = (int*)mxCalloc(nT,sizeof(*nF_HIT));
    nF_sketch = (int*)mxCalloc(nT,sizeof(*nF_sketch));
    nF_texture = (int*)mxCalloc(nT,sizeof(*nF_texture));
    nF_flatness = (int*)mxCalloc(nT,sizeof(*nF_flatness));
	nF_color = (int*)mxCalloc(nT,sizeof(*nF_color));
    for(int iT=0; iT<nT; ++iT)
    {
        nF_HIT[iT] = 0;
        const mxArray* f = mxGetField(all_HIT_sketch,iT,"selectedx");
        nF_sketch[iT] = 120; // mxGetM(f) * mxGetN(f);
        f = mxGetField(all_HIT_texture,iT,"selectedx");
        nF_texture[iT] = 90;//mxGetM(f) * mxGetN(f);
        f = mxGetField(all_HIT_flatness,iT,"selectedx");
        nF_flatness[iT] = 80; // mxGetM(f) * mxGetN(f);
		f = mxGetField(all_HIT_color,iT,"selectedx");
		nF_color[iT] = 0; //mxGetM(f) * mxGetN(f);
		colorHistVersion = (int)mxGetScalar( mxGetField(all_HIT_color,iT,"colorHistVersion") );
		
		switch(colorHistVersion)
		{
			case 0: nBinColor = 3; break;
			case 1: nBinColor = 24; break;
		}
		if( use_L2_as_texure_feature )
		{
			nF_HIT[iT] = nF_sketch[iT] + nF_texture[iT] + nF_flatness[iT] + nF_color[iT];
		}
		else
		{
			nF_HIT[iT] = nF_sketch[iT] + nF_texture[iT] * numOrient / 2 + nF_flatness[iT] + nF_color[iT] * nBinColor;
		}
    }
	
	if (display)
		mexPrintf("2");
	
    // compute orientation histogram maps
    int x,y,top,bottom,left,right;
    float* v = (float*)mxCalloc(numOrient,sizeof(float));

    int sub = (int)mxGetScalar( mxGetField(all_HIT_texture,0,"subsample") );
    int ax = (int)mxGetScalar( mxGetField(all_HIT_texture,0,"radius") );
    int ay = ax;
    int h = (int)mxGetScalar( mxGetField(all_HIT_texture,0,"halfFilterSize") );
    int sx = imHeight; int sy = imWidth;
    float** oriHist = (float**)mxCalloc( numOrient, sizeof(*oriHist) );
    for(int o = 0; o < numOrient; ++o)
    {
        oriHist[o] = (float*)mxCalloc( imWidth * imHeight, sizeof(**oriHist) );
        int nPixel = imWidth*imHeight;
        for( int j = 0; j < nPixel; ++j )
        {
            oriHist[o][j] = -1; // initialize the orientation histograms
                    // -1 means invalid value
        }
    }
    for(x=1+h+ax;x<sx-ax-h;x+=sub)
        for(y=1+ay+h;y<sy-ax-h;y+=sub)
        {
            top = x-ax;
            bottom = x+ax;
            left = y-ay;
            right = y+ay;
            poolOriHist(top,bottom,left,right,v,S1map);
            int here = px(x,y,imHeight);
            for(int o = 0; o < numOrient; ++o)
            {
                oriHist[o][here] = v[o];
            }
        }
        
	if (display)
		mexPrintf("3");
		
	// compute color histogram maps
	
	float *w = (float*)mxCalloc( nBinColor, sizeof(float) );
	ax = (int)mxGetScalar( mxGetField(all_HIT_color,0,"radius") );
	ay = ax;
	sub = (int)mxGetScalar( mxGetField(all_HIT_color,0,"subsample") );
	float **colorHist = (float**)mxCalloc( nBinColor, sizeof(*colorHist) );
	for(int o = 0; o < nBinColor; ++o)
    {
        colorHist[o] = (float*)mxCalloc( imWidth * imHeight, sizeof(**colorHist) );
        int nPixel = imWidth*imHeight;
        for( int j = 0; j < nPixel; ++j )
        {
            colorHist[o][j] = -1; // initialize the orientation histograms
                    // -1 means invalid value
        }
    }
	
	if (display)
		mexPrintf("4");
	
	for(x=1+ax;x<sx-ax;x+=sub)
	{
        for(y=1+ay;y<sy-ax;y+=sub)
        {
            top = x-ax;
            bottom = x+ax;
            left = y-ay;
            right = y+ay;
			poolColorHist(top,bottom,left,right,w,colorIm,colorHistVersion);
            for(int o = 0; o < nBinColor; ++o)
            {
                colorHist[o][x+y*imHeight] = w[o];
            }
        }
	}
	
	if (display)
		mexPrintf("5");
	
    // begin to collect features
    vec = (float**)mxCalloc(nT,sizeof(*vec));
    for(int iT=0; iT<nT; ++iT)
    {
        // for each template
        vec[iT] = (float*)mxCalloc(nF_HIT[iT],sizeof(**vec));
        int j = 0;
		
		const float* selectedx;
        const float* selectedy;
        const float* selectedo;
		
        // sketch:
		
        selectedx = (const float*) mxGetPr( mxGetField(all_HIT_sketch,iT,"selectedx") );
        selectedy = (const float*) mxGetPr( mxGetField(all_HIT_sketch,iT,"selectedy") );
        selectedo = (const float*) mxGetPr( mxGetField(all_HIT_sketch,iT,"selectedOrient") );

        for(int k = 0; k < nF_sketch[iT]; ++k)
        {
            int o = (int)selectedo[k];
            int x = (int)selectedx[k];
            int y = (int)selectedy[k];
			if (display>0)
			{
				// mexPrintf("#%d: x=%d, y=%d, o=%d\n",k+1,x,y,o);
			}
            vec[iT][j] = M1map[o][x+y*imHeight];
            ++j;
        }
		
		if (display)
			mexPrintf("6");
		
        // texture:
        selectedx = (const float*) mxGetPr( mxGetField(all_HIT_texture,iT,"selectedx") );
        selectedy = (const float*) mxGetPr( mxGetField(all_HIT_texture,iT,"selectedy") );
        const float* meanHist = (const float*) mxGetPr( mxGetField(all_HIT_texture,iT,"meanHist") );
        for(int k = 0; k < nF_texture[iT]; ++k )
        {
            int x = (int)selectedx[k];
            int y = (int)selectedy[k];
            // calculate L2 distance
            float dif = 0;
            for( int o = 0; o < numOrient; o+=2 )
            {
                float ho = oriHist[o][x+y*imHeight];
                if(ho<0)
                {
                    mexPrintf("warning: invalid oriHist\n");
                    ho = 0;
                }
                int ind = k + nF_texture[iT] * o;
				if( use_L2_as_texure_feature )
				{
					dif += ( ho-meanHist[ind] ) * ( ho-meanHist[ind] );
				}
				else
				{
					vec[iT][j] = ho * 5; ++j; // defult 3
					// vec[iT][j] = ho * ho * 1; ++j;
				}
            }
            if( use_L2_as_texure_feature )
			{
				vec[iT][j] = dif * 5;  ++j;
			}
        }
		
		if (display)
			mexPrintf("7");
		
        // flatness:
        selectedx = (const float*) mxGetPr( mxGetField(all_HIT_flatness,iT,"selectedx") );
        selectedy = (const float*) mxGetPr( mxGetField(all_HIT_flatness,iT,"selectedy") );
        for(int k = 0; k < nF_flatness[iT]; ++k )
        {
            int x = (int)selectedx[k];
            int y = (int)selectedy[k];
            vec[iT][j] = meanS1map[x+y*imHeight] * 3;
            ++j;
        }
		
		if (display)
			mexPrintf("8");
		
		// color
		
		selectedx = (const float*) mxGetPr( mxGetField(all_HIT_color,iT,"selectedx") );
        selectedy = (const float*) mxGetPr( mxGetField(all_HIT_color,iT,"selectedy") );
        meanHist = (const float*) mxGetPr( mxGetField(all_HIT_color,iT,"meanHist") );
        for(int k = 0; k < nF_color[iT]; ++k )
        {
            int x = (int)selectedx[k];
            int y = (int)selectedy[k];
            // calculate L2 distance
            float dif = 0;
            for( int o = 0; o < nBinColor; o+=2 )
            {
                float ho = colorHist[o][x+y*imHeight];
                if(ho<0)
                {
                    mexPrintf("warning: invalid colorHist\n");
                    ho = 0;
                }
                int ind = k + nF_color[iT] * o;
				if( use_L2_as_texure_feature )
				{
					dif += ( ho-meanHist[ind] ) * ( ho-meanHist[ind] );
				}
				else
				{
					vec[iT][j] = ho * 1; ++j;
					//vec[iT][j] = ho * ho * 12; ++j;
				}
            }
            if( use_L2_as_texure_feature )
			{
				vec[iT][j] = dif * 5;  ++j;
			}
        }
    }
}




void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    const mxArray *f;
    const mxArray* pA;
    mxArray* pA2;
    int nMap, i;

	
	
    /* input 0: number of quantized orientations */
    numOrient = (int)mxGetScalar(prhs[0]);

    /* input 1: SUM1 maps (multiscale, multi-orientation) */
    pA = prhs[1];
    nMap = mxGetM(pA) * mxGetN(pA);
    S1map = (const float**) mxCalloc( nMap, sizeof(*S1map) );
    for(i = 0; i < nMap; ++i)
    {
        f = mxGetCell( pA, i );
        S1map[i] = (const float*) mxGetPr( f );
        imHeight = mxGetM( f );
        imWidth = mxGetN(f);
    }

    /* input 2 : MAX1 maps (multiscale, multi-orientation) */
    pA = prhs[2];
    nMap = mxGetM(pA) * mxGetN(pA);
    M1map = (const float**) mxCalloc( nMap, sizeof(*M1map) );
    for(i = 0; i < nMap; ++i)
    {
        f = mxGetCell( pA, i );
        M1map[i] = (const float*) mxGetPr( f );
    }

    /* input 3: mean SUM1 map over orientations */
    meanS1map = (const float*) mxGetPr( prhs[3] );
	
	/* input 4: color images */
	pA = prhs[4];
	nMap = mxGetM(pA) * mxGetN(pA);
    colorIm = (const float**) mxCalloc( nMap, sizeof(*colorIm) );
    for(i = 0; i < nMap; ++i)
    {
        f = mxGetCell( pA, i );
        colorIm[i] = (const float*) mxGetPr( f );
    }

    /* input 5: HIT sketch features for multiple templates */
    all_HIT_sketch = prhs[5];
    nT = mxGetM(all_HIT_sketch) * mxGetN(all_HIT_sketch); /* number of templates */
	

    /* input 6: HIT texture features for multiple templates */
    all_HIT_texture = prhs[6];

    /* input 7: HIT texture features for multiple templates */
    all_HIT_flatness = prhs[7];
	
	/* input 8: HIT texture features for multiple templates */
    all_HIT_color = prhs[8];
	
	/* input 9: whether to display debugging information */
	display = (int)mxGetScalar( prhs[9] );
	
	if (display)
		mexPrintf("0");
    /* computation */
    Compute();
	
	if (display)
		mexPrintf("A\n");

    /* output 0: feature vectors organized as a cell array */
    mwSize dimsOutput[2];
    dimsOutput[0] = nT; dimsOutput[1] = 1;
    plhs[0] = mxCreateCellArray( 2, dimsOutput );
    for(int iT = 0; iT < nT; ++iT)
    {
        dimsOutput[0] = nF_HIT[iT]; dimsOutput[1] = 1;
        mxArray* pA = mxCreateNumericArray( 2, dimsOutput, mxSINGLE_CLASS, mxREAL );
        float* start_of_pr = (float*)mxGetData(pA);
        int bytes_to_copy = dimsOutput[0] * dimsOutput[1] * mxGetElementSize(pA);
        memcpy( start_of_pr, vec[iT], bytes_to_copy );
        mxSetCell( plhs[0], iT, pA );
    }

    /* output 1: number of total features per hybrid template */
    dimsOutput[0] = nT; dimsOutput[1] = 1;
    pA2 = mxCreateNumericArray( 2, dimsOutput, mxINT32_CLASS, mxREAL );
    memcpy( (int*)mxGetData(pA2), nF_HIT, dimsOutput[0] * dimsOutput[1] * mxGetElementSize(pA2) );
    plhs[1] = pA2;

    /* output 2: number of sketch features per hybrid template */
    pA2 = mxCreateNumericArray( 2, dimsOutput, mxINT32_CLASS, mxREAL );
    memcpy( (int*)mxGetData(pA2), nF_sketch, dimsOutput[0] * dimsOutput[1] * mxGetElementSize(pA2) );
    plhs[2] = pA2;

    /* output 3: number of texture features per hybrid template */
    pA2 = mxCreateNumericArray( 2, dimsOutput, mxINT32_CLASS, mxREAL );
    memcpy( (int*)mxGetData(pA2), nF_texture, dimsOutput[0] * dimsOutput[1] * mxGetElementSize(pA2) );
    plhs[3] = pA2;

    /* output 4: number of flatness features per hybrid template */
    pA2 = mxCreateNumericArray( 2, dimsOutput, mxINT32_CLASS, mxREAL );
    memcpy( (int*)mxGetData(pA2), nF_flatness, dimsOutput[0] * dimsOutput[1] * mxGetElementSize(pA2) );
    plhs[4] = pA2;
	
	/* output 5: number of color features per hybrid template */
    pA2 = mxCreateNumericArray( 2, dimsOutput, mxINT32_CLASS, mxREAL );
    memcpy( (int*)mxGetData(pA2), nF_color, dimsOutput[0] * dimsOutput[1] * mxGetElementSize(pA2) );
    plhs[5] = pA2;
}
