function test(LearningMethod)
% model.ActiveBasis.test_vs_all - testing on positive images and negative examples of other categories.
%
% LearningMethod: e.g. 'lrl2','svml2','svmlight'
% iRun: the number of cross validation run.
% categories: the models (organized by category) to test.

smallTestSize = false;
toReloadFeature = 1;

modelName = 'HIT';
windowSize = feature.getWindowSize;
featureVersion = 'GaborS1';
Upperbound = feature.getGaborResponseUpperbound;

% load all the trained models
if strcmp(LearningMethod(1:2),'lr')
	load(sprintf('Result/%s/model_%s.mat',modelName,LearningMethod),'lr_model');
	weights = lr_model.w(:);
	clear lr_model;
elseif strcmp(LearningMethod(1:3),'svm')
	load(sprintf('Result/%s/model_%s.mat',modelName,LearningMethod),'svm_model');
	weights = svm_model.w(:);
	clear svm_model;
elseif strcmp(LearningMethod(1:6),'libsvm')
	load(sprintf('Result/%s/model_%s.mat',modelName,LearningMethod),'model');
	weights = model.w(:);
	clear model;
end

filename = sprintf( 'Result/%s/hit.mat', modelName );
load( filename, 'Lrange', 'Orange', 'windowSize',...
        'HIT_sketch', 'HIT_texture', 'HIT_flatness', 'HIT_color' );

% test on multi-categories
fprintf(1,'testing ---- \n');
tic

% load the template
filename = sprintf( 'Result/%s/hit.mat', modelName );
load( filename );

tic

if toReloadFeature

	Xpos = []; % data matrix
	imageSet = 'Test/pos';
	files = dir( sprintf('Feature/%s/%s/*.mat',featureVersion,imageSet) );
	n = length(files);
	if smallTestSize
		n = min(100,n);
	end
	
	i = 1;
	featurePath = sprintf('Feature/%s/%s/%05d.mat',featureVersion,imageSet,i);
	load(featurePath);
	featurePath2 = sprintf('Feature/%s/%s/%05d.mat','HoG',imageSet,i); % to get the image patch
	load(featurePath2,'patch');
	
	model.HIT.load_feature;
	D = length(feavec{1});
	Xpos = zeros(n,D);
	Xpos(1,:) = feavec{1}(:);
	
	for i = 2:n
		featurePath = sprintf('Feature/%s/%s/%05d.mat',featureVersion,imageSet,i);
		load(featurePath);
		model.HIT.load_feature;
		Xpos(i,:) = feavec{1}(:);
	end
	
	feature_cache = sprintf('Result/%s/test_fea.mat',modelName);
	save(feature_cache,'Xpos');
	
else

	feature_cache = sprintf('Result/%s/test_fea.mat',modelName);
	load(feature_cache,'Xpos');
	
end
pos_score = [Xpos,ones(size(Xpos,1),1)] * weights; % N by D+1, times D+1 by 1

clear Xpos

if toReloadFeature
	Xneg = []; % data matrix
	imageSet = 'Test/neg';
	files = dir( sprintf('Feature/%s/%s/*.mat',featureVersion,imageSet) );
	n = length(files);
	if smallTestSize
		n = min(n,1000);
	else
		n = min(n,10000);
	end
	
	i = 1;
	featurePath = sprintf('Feature/%s/%s/%05d.mat',featureVersion,imageSet,i);
	load(featurePath);
	model.HIT.load_feature;
	D = length(feavec{1});
	Xneg = zeros(n,D);
	Xneg(1,:) = feavec{1}(:);
	
	for i = 2:n
		featurePath = sprintf('Feature/%s/%s/%05d.mat',featureVersion,imageSet,i);
		load(featurePath);
		model.HIT.load_feature;
		Xneg(i,:) = feavec{1}(:);
	end
	feature_cache = sprintf('Result/%s/test_fea.mat',modelName);
	save(feature_cache,'Xneg','-append');
else
	feature_cache = sprintf('Result/%s/test_fea.mat',modelName);
	load(feature_cache,'Xneg');
end
neg_score = [Xneg,ones(size(Xneg,1),1)] * weights; % N by D+1, times D+1 by 1

out_file = sprintf('Result/%s/testing_scores_%s.mat',modelName,LearningMethod);
save( out_file, 'pos_score', 'neg_score' );

fprintf(1,'takes %.1f sec\n',toc);


