function test(LearningMethod)
% model.HoGSimple.test - testing on positive images and negative examples of other categories.
%
% LearningMethod: e.g. 'lrl2','svml2','svmlight'
% iRun: the number of cross validation run.
% categories: the models (organized by category) to test.

smallTestSize = false;

modelName = 'HoGSimple';
windowSize = feature.getWindowSize;
sbin = feature.HoG_binsize;
featureVersion = 'HoG';


% make sure the result folder is created
if ~exist(sprintf('Result/%s',modelName),'dir')
	mkdir(sprintf('Result/%s',modelName));
end

% load all the trained models
weights = [];
if strcmp(LearningMethod(1:2),'lr')
	load(sprintf('Result/%s/model_%s.mat',modelName,LearningMethod),'lr_model');
	weights = [weights,lr_model.w(:)];
elseif strcmp(LearningMethod(1:3),'svm')
	load(sprintf('Result/%s/model_%s.mat',modelName,LearningMethod),'svm_model');
	weights = [weights,svm_model.w(:)];
	clear svm_model;
elseif strcmp(LearningMethod(1:6),'libsvm')
	load(sprintf('Result/%s/model_%s.mat',modelName,LearningMethod),'model');
	weights = [weights,model.w(:)];
	clear model;
end

% test on images
fprintf(1,'testing ---- ');
tic
Xpos = []; % data matrix
imageSet = 'Test/pos';
files = dir( sprintf('Feature/%s/%s/*.mat',featureVersion,imageSet) );
n = length(files);
if smallTestSize
	n = min(n,100);
end
for i = 1:n
    featurePath = sprintf('Feature/%s/%s/%05d.mat',featureVersion,imageSet,i);
    load(featurePath);
    D = numel(HoGmap);
    if size(Xpos,1) == 0
    	Xpos = zeros(n,D);
    	Xpos(1,:) = HoGmap(:);
    else
    	Xpos(i,:) = HoGmap(:);
    end
end
pos_score = [Xpos,ones(size(Xpos,1),1)] * weights; % N by D+1, times D+1 by 1

clear Xpos
Xneg = []; % data matrix
imageSet = 'Test/neg';
files = dir( sprintf('Feature/%s/%s/*.mat',featureVersion,imageSet) );
n = length(files);
if smallTestSize
	n = min(n,1000);
else
	n = min(n,10000);
end
for i = 1:n
    featurePath = sprintf('Feature/%s/%s/%05d.mat',featureVersion,imageSet,i);
    load(featurePath,'HoGmap');
    D = numel(HoGmap);
    if size(Xneg,1) == 0
    	Xneg = zeros(n,D);
    	Xneg(1,:) = HoGmap(:);
    else
    	Xneg(i,:) = HoGmap(:);
    end
end
neg_score = [Xneg,ones(size(Xneg,1),1)] * weights; % N by D+1, times D+1 by 1

out_file = sprintf('Result/%s/testing_scores_%s.mat',modelName,LearningMethod);
save( out_file, 'pos_score', 'neg_score' );

